// HtmlView2.cpp : implementation file
//

#include "stdafx.h"
#include "HtmlView2.h"

#include <comdef.h>
#include <mshtml.h>
#include <tchar.h>
#include <wininet.h>
#include <intshcut.h>
#include <atlconv.h>

#pragma comment( lib, "wininet.lib" )

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const GUID GUID_IWebBrowser = {
	0xED016940,
	0xBD5B,
	0x11CF, {
		0xBA,
		0x4E,
		0x00,
		0xC0,
		0x4F,
		0xD7,
		0x08,
		0x16
	}
};

const UINT CHtmlView2::CF_HTML = ::RegisterClipboardFormat( _T("HTML Format") );

/////////////////////////////////////////////////////////////////////////////
// CHtmlView2

IMPLEMENT_DYNCREATE(CHtmlView2, CHtmlView)

CHtmlView2::CHtmlView2()
	: m_FontSize( MediumSize )
{
	//{{AFX_DATA_INIT(CHtmlView2)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CHtmlView2::~CHtmlView2()
{
}

BEGIN_MESSAGE_MAP(CHtmlView2, CHtmlView)
	//{{AFX_MSG_MAP(CHtmlView2)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHtmlView2 implementation

HRESULT CHtmlView2::ExecCommandHelper(DWORD dwCommand)
{
	IDispatch * pDisp = NULL;
	IOleCommandTarget * pCmdTarget = NULL;
	HRESULT hr;

	pDisp = GetHtmlDocument();
	ASSERT(pDisp);
#ifndef _DEBUG
	if (!pDisp)
		return E_FAIL;
#endif

	hr = pDisp->QueryInterface( IID_IOleCommandTarget,
		reinterpret_cast< void ** > (&pCmdTarget) );
	pDisp->Release();

	ASSERT(pCmdTarget);
#ifndef _DEBUG
	if (!pCmdTarget)
		return hr;
#endif

	hr = pCmdTarget->Exec( &GUID_IWebBrowser, dwCommand, 0, NULL, NULL );

	return hr;
}

bool CHtmlView2::InvokeFind()
{
	return SUCCEEDED(ExecCommandHelper( FindCommand )) ? true : false;
}

bool CHtmlView2::InvokeViewSource()
{
	return SUCCEEDED(ExecCommandHelper( ViewSourceCommand )) ? true : false;
}

bool CHtmlView2::InvokeInternetOptions()
{
	return SUCCEEDED(ExecCommandHelper( OptionsCommand )) ? true : false;
}

CString CHtmlView2::GetDocumentSelection()
{
	LPDISPATCH lpDispatch = NULL;
	IHTMLDocument2 * lpHtmlDoc = NULL;
	IHTMLSelectionObject * lpSelection = NULL;
	IHTMLTxtRange * lpTxtRange = NULL;
	BSTR bstr;
	CString strResult;
	HRESULT hr = E_FAIL;

	lpDispatch = GetHtmlDocument();

	if (lpDispatch)
	{
		hr = lpDispatch->QueryInterface( IID_IHTMLDocument2, 
			(void**) &lpHtmlDoc );

		lpDispatch->Release();
		lpDispatch = NULL;

		if (SUCCEEDED(hr))
		{
			hr = lpHtmlDoc->get_selection( &lpSelection );

			if (SUCCEEDED(hr))
			{
				hr = lpSelection->createRange( &lpDispatch );

				if (SUCCEEDED(hr))
				{
					hr = lpDispatch->QueryInterface( IID_IHTMLTxtRange,
						(void**) &lpTxtRange );

					lpDispatch->Release();
					lpDispatch = NULL;

					if (SUCCEEDED(hr))
					{
						hr = lpTxtRange->get_htmlText( &bstr );

						if (SUCCEEDED(hr))
						{
							try
							{
								_bstr_t tmpStr( bstr );
								strResult = (const TCHAR *) tmpStr;
							}
							catch (_com_error err)
							{
								MessageBox( err.ErrorMessage(),
									_T("Error"),
									MB_OK | MB_ICONSTOP | MB_APPLMODAL );
								strResult.Empty();
							}

							::SysFreeString( bstr );
						}

						lpTxtRange->Release();
					}
				}

				lpSelection->Release();
			}

			lpHtmlDoc->Release();
		}
	}
	return strResult;
}

bool CHtmlView2::IsGlobalOffline()
{
    DWORD dwState = 0; 
    DWORD dwSize = sizeof(DWORD);
    bool bRet = false;

    if(::InternetQueryOption( NULL, 
        INTERNET_OPTION_CONNECTED_STATE, &dwState, &dwSize ))
    {
        if(dwState & INTERNET_STATE_DISCONNECTED_BY_USER)
            bRet = true;
    }
    return bRet; 
}

bool CHtmlView2::AddToFavorites()
{
	IShellUIHelper * pShellUIHelper = NULL;
	HRESULT hr;
	bool bResult = false;

	hr = ::CoCreateInstance( CLSID_ShellUIHelper,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_IShellUIHelper,
		(void **) &pShellUIHelper );

	if (SUCCEEDED(hr))
	{
		_bstr_t bstrURL( (LPCTSTR) GetLocationURL() );
		BSTR bstrName = GetLocationName().AllocSysString();
		_variant_t vtTitle( bstrName );
		bResult = SUCCEEDED(pShellUIHelper->AddFavorite( bstrURL, &vtTitle )) ? true : false;
		pShellUIHelper->Release();
		::SysFreeString( bstrName );
	}

	return bResult;
}

bool CHtmlView2::OrganizeFavorites()
{
	typedef UINT (CALLBACK* LPFNORGFAV)(HWND, LPTSTR);

	bool bResult = false;

	HMODULE hMod = ::GetModuleHandle( _T("shdocvw.dll") );

	if (hMod == NULL)
	{
		hMod = ::LoadLibrary( _T("shdocvw.dll") );

		if (hMod == NULL)
		{
			MessageBox( _T("The dynamic link library ShDocVw.DLL cannot be found."),
				_T("Error"), MB_OK | MB_ICONSTOP );
			return false;
		}
		LPFNORGFAV lpfnDoOrganizeFavDlg = (LPFNORGFAV)
			::GetProcAddress( hMod, "DoOrganizeFavDlg" );

		if (lpfnDoOrganizeFavDlg == NULL)
		{
			MessageBox( _T("The entry point DoOrganizeFavDlg cannot be found\n")
				_T("in the dynamic link library ShDocVw.DLL."),
				_T("Error"), MB_OK | MB_ICONSTOP );
			return false;
		}

		TCHAR szPath [ MAX_PATH ];
		HRESULT hr;

		hr = ::SHGetSpecialFolderPath( m_hWnd, szPath, CSIDL_FAVORITES, TRUE );
		if (FAILED(hr))
		{
			MessageBox( _T("The path of the Favorites folder cannot be found."),
				_T("Error"), MB_OK | MB_ICONSTOP );
			return false;
		}

		bResult = (*lpfnDoOrganizeFavDlg) ( m_hWnd, szPath ) ? true : false;

		::FreeLibrary( hMod );
	}
	else
	{
		LPFNORGFAV lpfnDoOrganizeFavDlg = (LPFNORGFAV)
			::GetProcAddress( hMod, "DoOrganizeFavDlg" );

		if (lpfnDoOrganizeFavDlg == NULL)
		{
			MessageBox( _T("The entry point DoOrganizeFavDlg cannot be found\n")
				_T("in the dynamic link library ShDocVw.DLL."),
				_T("Error"), MB_OK | MB_ICONSTOP );
			return false;
		}

		TCHAR szPath [ MAX_PATH ];
		HRESULT hr;

		hr = ::SHGetSpecialFolderPath( m_hWnd, szPath, CSIDL_FAVORITES, TRUE );
		if (FAILED(hr))
		{
			MessageBox( _T("The path of the Favorites folder cannot be found."),
				_T("Error"), MB_OK | MB_ICONSTOP );
			return false;
		}

		bResult = (*lpfnDoOrganizeFavDlg) ( m_hWnd, szPath ) ? true : false;
	}

	return bResult;
}

bool CHtmlView2::SaveInternetShortcut(LPCTSTR pszFileName,
									  LPCTSTR pszURL,
									  LPCTSTR pszDescription,
									  LPCTSTR pszIconFile,
									  int nIndex)
{
	bool bResult = false;

	IUniformResourceLocator * pURL = NULL;
	IPersistFile * pPF = NULL;
	IShellLink * pSL = NULL;
	HRESULT hr;

	USES_CONVERSION;

	::CoInitialize( NULL );

	hr = ::CoCreateInstance( CLSID_InternetShortcut,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_IUniformResourceLocator,
		(void **) &pURL );

	if (SUCCEEDED(hr))
	{
		hr = pURL->QueryInterface( IID_IPersistFile,
			(void **) &pPF );

		if (SUCCEEDED(hr))
		{
			hr = pURL->QueryInterface( IID_IShellLink,
				(void **) &pSL );

			if (SUCCEEDED(hr))
			{
				hr = pURL->SetURL( pszURL, IURL_SETURL_FL_USE_DEFAULT_PROTOCOL );

				if (SUCCEEDED(hr))
				{
					hr = pSL->SetIconLocation( pszIconFile, nIndex );

					if (SUCCEEDED(hr))
					{
						hr = pSL->SetDescription( pszDescription );

						if (SUCCEEDED(hr))
						{
							hr = pPF->Save( T2COLE(pszFileName), TRUE );

							if (SUCCEEDED(hr))
							{
								bResult = true;
							}
						}
					}
				}

				pSL->Release();
			}

			pPF->Release();
		}

		pURL->Release();
	}

	return bResult;
}

bool CHtmlView2::CreateShortcut()
{
	CString strTitle;
	strTitle = GetLocationName();

	// remove illegal characters
	strTitle.Replace( _T('\\'), _T('-') );
	strTitle.Replace( _T('/'), _T('-') );
	strTitle.Replace( _T('*'), _T('-') );
	strTitle.Replace( _T('?'), _T('-') );
	strTitle.Replace( _T('|'), _T('-') );
	strTitle.Replace( _T('<'), _T('-') );
	strTitle.Replace( _T('>'), _T('-') );
	strTitle.Replace( _T(':'), _T('-') );
	strTitle.Replace( _T('\"'), _T('-') );

	TCHAR szPath [ MAX_PATH ];
	IMalloc * pMalloc = NULL;
	HRESULT hr;
	LPITEMIDLIST pidl = NULL;

	BROWSEINFO bi;
	bi.hwndOwner = m_hWnd;
	bi.pidlRoot = NULL;
	bi.pszDisplayName = szPath;
	bi.lpszTitle = _T("Select the folder in which you want the shortcut created.");
	bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE;
	bi.lpfn = NULL;
	bi.lParam = 0;
	bi.iImage = 0;

	::CoInitialize( NULL );

	hr = ::SHGetMalloc( &pMalloc );
	if (FAILED(hr))
		return false;

	pidl = ::SHBrowseForFolder( &bi );

	if (!::SHGetPathFromIDList( pidl, szPath ))
	{
		pMalloc->Free( pidl );
		pMalloc->Release();
		return false;
	}


	TCHAR * pc = szPath;
	while (*pc ++);
	pc -= 2;
	if (*pc != _T('\\'))
	{
		pc ++;
		*pc = _T('\\');
		pc ++;
		*pc = _T('\0');
	}
	::_tcscat( szPath, _T("Shortcut to ") );
	::_tcscat( szPath, strTitle );
	::_tcscat( szPath, _T(".url") );

	if (!SaveInternetShortcut( szPath, GetLocationURL(), strTitle ))
		return false;

	// cleanup
	pMalloc->Free( pidl );
	pMalloc->Release();

	return true;
}

void CHtmlView2::Copy()
{
	ExecWB( OLECMDID_COPY, OLECMDEXECOPT_DODEFAULT, NULL, NULL );
}

void CHtmlView2::Cut()
{
	ExecWB( OLECMDID_CUT, OLECMDEXECOPT_DODEFAULT, NULL, NULL );
}

void CHtmlView2::Paste()
{
	ExecWB( OLECMDID_PASTE, OLECMDEXECOPT_DODEFAULT, NULL, NULL );
}

void CHtmlView2::PasteSmart()
{
	CString strActiveElem = GetActiveElement();
	if (strActiveElem.Find( _T("INPUT") ) >= 0 ||
		strActiveElem.CompareNoCase( _T("TEXTAREA") ) == 0)
	{
		ExecWB( OLECMDID_PASTE, OLECMDEXECOPT_DODEFAULT, NULL, NULL );
	}
}

CString CHtmlView2::GetActiveElement()
{
	CString strTag;
	IHTMLElement * pHTMLElem = NULL;
	IHTMLInputElement * pInputElem = NULL;
	IHTMLTextAreaElement * pTextAreaElem = NULL;
	IHTMLAnchorElement * pLink = NULL;
	IHTMLAreaElement * pAreaLink = NULL;
	HRESULT hr;
	BSTR bstr = NULL;

	if (!GetActiveElement( &pHTMLElem ))
		return strTag;

	ASSERT(pHTMLElem);

	if (pHTMLElem != NULL)
	{
		hr = pHTMLElem->QueryInterface( IID_IHTMLInputElement,
			(void **) &pInputElem );

		if (hr == S_OK && pInputElem != NULL)
		{
			strTag = _T("INPUT");

			hr = pInputElem->get_type( &bstr );

			if (hr == S_OK)
			{
				CString strTemp = bstr;
				if (strTemp.CompareNoCase( _T("file") ) == 0)
					strTag += _T("-FILE");
				else if (strTemp.CompareNoCase( _T("password") ) == 0)
					strTag += _T("-PASSWORD");
				else if (strTemp.CompareNoCase( _T("text") ) == 0)
					strTag += _T("-TEXT");

				::SysFreeString( bstr );
			}

			pInputElem->Release();
		}
		else
		{
			hr = pHTMLElem->QueryInterface( IID_IHTMLTextAreaElement,
				(void **) &pTextAreaElem );

			if (hr == S_OK && pTextAreaElem != NULL)
			{
				strTag = _T("TEXTAREA");

				pTextAreaElem->Release();
			}
			else
			{
				hr = pHTMLElem->get_tagName( &bstr );

				if (hr == S_OK)
				{
					strTag = bstr;
					::SysFreeString( bstr );
				}
			}
		}

		pHTMLElem->Release();
	}

	strTag.MakeUpper();
	return strTag;
}

BOOL CHtmlView2::UpdateCopyHelper()
{
	return !(GetDocumentSelection().IsEmpty());
}

BOOL CHtmlView2::UpdateCutHelper()
{
	CString strTag = GetActiveElement();
	return ((!GetDocumentSelection().IsEmpty()) &&
		((strTag.Find( _T("INPUT-") ) >= 0) ||
		(strTag.CompareNoCase( _T("TEXTAREA") ) == 0)));
}

BOOL CHtmlView2::UpdatePasteHelper()
{
	CString strTag = GetActiveElement();
	return ((::IsClipboardFormatAvailable( CF_TEXT ) ||
		::IsClipboardFormatAvailable( CF_UNICODETEXT ) ||
		::IsClipboardFormatAvailable( CF_OEMTEXT ) ||
		::IsClipboardFormatAvailable( CF_HTML )) &&
		((strTag.Find( _T("INPUT-") ) >= 0) ||
		(strTag.CompareNoCase( _T("TEXTAREA") ) == 0)));
}

void CHtmlView2::SelectAll()
{
	ExecWB( OLECMDID_SELECTALL, OLECMDEXECOPT_DODEFAULT, NULL, NULL );
}

void CHtmlView2::Properties()
{
	ExecWB(OLECMDID_PROPERTIES, OLECMDEXECOPT_DODEFAULT, NULL, NULL);
}

void CHtmlView2::SetFontSize(long lSize)
{
	COleVariant vaZoomFactor( lSize );
	m_FontSize = (enum FontSize) lSize;

	ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER, &vaZoomFactor, NULL);
}

void CHtmlView2::SetFontSizeSmallest()
{
	SetFontSize( SmallestSize );
}

void CHtmlView2::SetFontSizeSmall()
{
	SetFontSize( SmallSize );
}

void CHtmlView2::SetFontSizeMedium()
{
	SetFontSize( MediumSize );
}

void CHtmlView2::SetFontSizeLarge()
{
	SetFontSize( LargeSize );
}

void CHtmlView2::SetFontSizeLargest()
{
	SetFontSize( LargestSize );
}

enum CHtmlView2::FontSize CHtmlView2::GetCurrentFontSize()
{
	return m_FontSize;
}

BOOL CHtmlView2::FontSizeSmallestUpdateHelper()
{
	return GetCurrentFontSize() == SmallestSize;
}

BOOL CHtmlView2::FontSizeSmallUpdateHelper()
{
	return GetCurrentFontSize() == SmallSize;
}

BOOL CHtmlView2::FontSizeMediumUpdateHelper()
{
	return GetCurrentFontSize() == MediumSize;
}

BOOL CHtmlView2::FontSizeLargeUpdateHelper()
{
	return GetCurrentFontSize() == LargeSize;
}

BOOL CHtmlView2::FontSizeLargestUpdateHelper()
{
	return GetCurrentFontSize() == LargestSize;
}

void CHtmlView2::OnInitialUpdate() 
{
	CHtmlView::OnInitialUpdate();

	SetOffline( IsGlobalOffline() );
}

void CHtmlView2::ToggleWorkOffline()
{
	SetOffline( !GetOffline() );
}

BOOL CHtmlView2::WorkOfflineUpdateHelper()
{
	return GetOffline();
}

void CHtmlView2::SavePage()
{
	ExecWB( OLECMDID_SAVEAS, OLECMDEXECOPT_PROMPTUSER, NULL, NULL );
}

bool CHtmlView2::GetActiveElement(IHTMLElement **ppActiveElem)
{
	IDispatch * pDisp = NULL;
	IHTMLDocument2 * pHTMLDoc = NULL;
	IHTMLElement * pHTMLElem = NULL;
	HRESULT hr;
	bool bResult = false;

	pDisp = GetHtmlDocument();
	if (pDisp != NULL)
	{
		hr = pDisp->QueryInterface( IID_IHTMLDocument2, (void **) &pHTMLDoc );

		pDisp->Release();
		pDisp = NULL;

		if (pHTMLDoc != NULL)
		{
			hr = pHTMLDoc->get_activeElement( &pHTMLElem );

			if (hr == S_OK && pHTMLElem != NULL)
			{
				*ppActiveElem = pHTMLElem;
				bResult = true;
			}

			pHTMLDoc->Release();
		}
	}

	return bResult;
}

bool CHtmlView2::SetAsDesktopItem(IHTMLElement * pActiveElem)
{
	IShellUIHelper * pShellUIHelper = NULL;
	HRESULT hr;

	ASSERT(pActiveElem);
	if (!pActiveElem)
		return false;

	hr = ::CoCreateInstance( CLSID_ShellUIHelper,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_IShellUIHelper,
		(void **) &pShellUIHelper );

	if (SUCCEEDED(hr))
	{
		_bstr_t bstrType( _T("website") );
		IHTMLAnchorElement * pAnchorElem = NULL;
		IHTMLImgElement * pImgElem = NULL;

		BSTR bstrURL;

		if (SUCCEEDED(pActiveElem->QueryInterface( IID_IHTMLAnchorElement,
			(void **) &pAnchorElem )))
		{
			if (FAILED(pAnchorElem->get_href( &bstrURL )))
				return false;
		}
		else if (SUCCEEDED(pActiveElem->QueryInterface( IID_IHTMLImgElement,
			(void ** ) &pImgElem )))
		{
			if (FAILED(pImgElem->get_src( &bstrURL )))
				return false;
		}
		else
		{
			bstrURL = GetLocationURL().AllocSysString();
		}

		hr = pShellUIHelper->AddDesktopComponent( bstrURL,
			bstrType, NULL, NULL, NULL, NULL );
		pShellUIHelper->Release();
		if (bstrURL)
			::SysFreeString( bstrURL );
	}

	return SUCCEEDED(hr) ? true : false;
}

bool CHtmlView2::SetAsDesktopItem()
{
	IHTMLElement * pActiveElem = NULL;
	bool bResult = false;
	if (GetActiveElement( &pActiveElem ))
	{
		bResult = SetAsDesktopItem( pActiveElem );
		pActiveElem->Release();
	}
	return bResult;
}

void CHtmlView2::Undo()
{
	ExecWB( OLECMDID_UNDO, OLECMDEXECOPT_DODEFAULT, NULL, NULL );
}

bool CHtmlView2::StartWebBrowser()
{
	return ::ShellExecute( ::AfxGetMainWnd()->m_hWnd,
		_T("open"),
		_T("about:blank"),
		NULL,
		NULL,
		SW_SHOWNORMAL ) ? true : false;
}
